import 'package:iform/component/iform_common_json.dart';
import 'package:nui_core/data/nui_ent.dart';
import 'package:nui_core/datetime/nui_dt_util.dart';
import 'package:nui_core/nui_core.dart';

enum IFDatePickerType{
  DATE,
  BIRTHDAY_AGE,
  DATE_TIME,
  DATE_RANGE,
  BIRTHDAY
}
extension IFDatePickerTypeExt on IFDatePickerType{
  int value(){
    switch(this){
      case IFDatePickerType.DATE: return 1;
      case IFDatePickerType.BIRTHDAY_AGE: return 2;
      case IFDatePickerType.DATE_TIME: return 3;
      case IFDatePickerType.DATE_RANGE: return 4;
      case IFDatePickerType.BIRTHDAY: return 5;
    }
  }
}

typedef IFDateTimeOnValueChange = Function(String value);

class IFDateTimePickerJson extends NUIEnt<IFDateTimePickerJson> with IFCommonJson{
  IFDateTimePickerJson.empty();
  String dateTimeDisplayFormat;
  int dateTimePickerType;
  Function onFocus;
  IFDateTimeOnValueChange onValueChange;
  String minDate; // yyyy-MM-dd, CurrentDate, CurrentDate+1y, CurrentDate+1m, CurrentDate+1d, CurrentDate-1y, CurrentDate-1m, CurrentDate-1d
  String maxDate;
  String validateErrMsg;

  @override
  NUIEntMapper<IFDateTimePickerJson> mapper() => IFDateTimePickerJsonEnt();

  DateTime getDateFilterMargin(String filter){
    final regex = RegExp("[0-9]");
    final filterFormat = filter.replaceAll(regex,"x");

    if(match(filterFormat, "xxxx-xx-xx")){
      //The filter is an exact date
      return stringToDate(filter);
    }
    if(match(filterFormat, "CurrentDate")){
      return DateTime.now();
    }

    //Calculate the date difference margin for filter
    final processedFilter = filter.replaceAll("CurrentDate", "");
    try{
      final positiveNegative = processedFilter[0];
      final durationUnit = processedFilter[processedFilter.length-1];
      final durationValue = int.parse(processedFilter.substring(1, processedFilter.length - 1));

      bool isAdd = positiveNegative == '+';
      bool isMinus = positiveNegative == '-';

      Duration durationRange;
      if(durationUnit == 'y') durationRange = Duration(days: durationValue * 365);
      if(durationUnit == 'm') durationRange = Duration(days: durationValue * 30);
      if(durationUnit == 'd') durationRange = Duration(days: durationValue);

      if(durationRange == null) return null; //Incorrect unit used

      if(isAdd){
        //Days after today
        return DateTime.fromMillisecondsSinceEpoch(DateTime.now().millisecondsSinceEpoch + durationRange.inMilliseconds);
      }
      else if(isMinus){
        //Days before today
        return DateTime.fromMillisecondsSinceEpoch(DateTime.now().millisecondsSinceEpoch - durationRange.inMilliseconds);
      }
    }catch(e){
      logNUI("IFDateTimePickerJson", "Failed to process the date filter with error : $e");
    }
    return null;
  }

  DateTime getMinimumDate(){
    return isNullOrEmpty(minDate) ? null : getDateFilterMargin(minDate);
  }

  DateTime getMaximumDate(){
    return isNullOrEmpty(maxDate) ? null : getDateFilterMargin(maxDate);
  }
}

class IFDateTimePickerJsonEnt extends NUIEntMapper<IFDateTimePickerJson>{
  @override
  String entityName() => "IFDateTimePicker";

  @override
  List<NUIEntField> fields() {
    final fields = IFCommonJson.dbFields<IFDateTimePickerJson>();
    fields.add(NUIEntField<IFDateTimePickerJson>(name: "dateTimeDisplayFormat", type: NUIEntType.STRING, setter: (data, value) => data.dateTimeDisplayFormat = value, getter: (data) => data.dateTimeDisplayFormat));
    fields.add(NUIEntField<IFDateTimePickerJson>(name: "dateTimePickerType", type: NUIEntType.INTEGER, setter: (data, value) => data.dateTimePickerType = value, getter: (data) => data.dateTimePickerType));
    fields.add(NUIEntField<IFDateTimePickerJson>(name: "minDate", type: NUIEntType.STRING, setter: (data, value) => data.minDate = value, getter: (data) => data.minDate));
    fields.add(NUIEntField<IFDateTimePickerJson>(name: "maxDate", type: NUIEntType.STRING, setter: (data, value) => data.maxDate = value, getter: (data) => data.maxDate));
    fields.add(NUIEntField<IFDateTimePickerJson>(name: "validateErrMsg", type: NUIEntType.STRING, setter: (data, value) => data.validateErrMsg = value, getter: (data) => data.validateErrMsg));
    return fields;
  }

  @override
  IFDateTimePickerJson newInstance() => IFDateTimePickerJson.empty();
}

