import 'dart:convert';

import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:iform/component/iform_body_json.dart';
import 'package:iform/renderengine/iform_render_engine.dart';
import 'package:nui_core/nui_core.dart';
import 'package:nui_form/nuif_form.dart';

void showSampleIForm(BuildContext context, {String title, bool showStatusBar: true, IFBodyJson body, String bodyJson}){
  showNUIDialog(context: context, content: (context) => IFPreviewUI(title: title, showStatusBar: showStatusBar, body: body, bodyJson: bodyJson));
}

class IFPreviewUI extends StatefulWidget {
  IFPreviewUI({this.title, this.showStatusBar: false, this.bodyJson, this.body});
  final bool showStatusBar;
  final String title;
  final IFBodyJson body;
  final String bodyJson;
  @override
  _IFPreviewUIState createState() => _IFPreviewUIState();
}

class _IFPreviewUIState extends State<IFPreviewUI> {
  @override
  Widget build(BuildContext context) {
    final children = List<Widget>();
    if(widget.showStatusBar == true) {
      children.add(NUIStatusBar(color: NUIColors.NUIShadowBlackTransparent));
    }
    children.add(NUIToolbar(
      content: Text(widget.title ?? "IForm Preview", style: TextStyle(fontSize: 14, color: NUIColors.NUITextBlack)),
      leftButton: NUIToolbarIconButton(
          image: Icon(Icons.arrow_back_ios),
          color: NUIColors.NUITextBlack,
          radius: 20.0,
          padding: 10,
          onTap: (){
            NUINavigator.pop(context);
          }
      ),
      titlePadding: 10,
      decoration: BoxDecoration(
        color: NUIColors.NUIWhite,
      ),
    ));

    final toolbar = Container(
      decoration: NUIBoxDecoration(
          shadowRadius: 10,
          shadowColor: NUIColors.NUIShadowColorBrightSolid,
          color: NUIColors.NUIWhite
      ),
      child: Column(
          mainAxisSize: MainAxisSize.max,
          children: children
      ),
    );

    final renderer = IFRenderEngineBuilder()
      .body(widget.body)
      .bodyJson(widget.bodyJson)
      .build();

    final form = renderer.renderForm(textChangeListener: (id, value, form){
      logNUI("IFormPreview", "Text changed on $id, cur value : $value");
    }, buttonClickListener: (id, clickType, form){
      logNUI("IFormPreview", "Button clicked on $id, click type : $clickType");
      final validated = form.validate(false);
      final values = form.extractValues();
      logNUI("IFormPreviuew", "${jsonEncode(values)}");
    }, itemSelectListener: (id, selection, selected, form){
      logNUI("IFormPreview", "Item selected on $id, cur value : ${selection.value}, selected : $selected");
    });

    return Scaffold(
      body: NUIColumn(
          children: [
            toolbar,
            Expanded(
              child: Container(
                padding: EdgeInsets.only(left: 15, right: 15),
                color: NUIColors.NUIWhite,
                child: ListView(
                  children: [
                    NUIFormContent(
                        form: form
                    ),
                  ]
                ),
              ),
            ),
          ],
      ),
    );
  }
}
