import 'dart:convert';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:iform/component/iform_body_json.dart';
import 'package:iform/component/iform_button_row_json.dart';
import 'package:iform/component/iform_checkgroup_json.dart';
import 'package:iform/component/iform_common_json.dart';
import 'package:iform/component/iform_datetime_picker_json.dart';
import 'package:iform/component/iform_dropdown_json.dart';
import 'package:iform/component/iform_enum.dart';
import 'package:iform/component/iform_label_json.dart';
import 'package:iform/component/iform_singleline_text_json.dart';
import 'package:iform/iform.dart';
import 'package:iform/model/iform_selection.dart';
import 'package:iform/style/iform_styling.dart';
import 'package:nui_core/code/nui_code_util.dart';
import 'package:nui_core/color/nui_colors.dart';
import 'package:nui_core/data/nui_data_util.dart';
import 'package:nui_core/data/nui_ent.dart';
import 'package:nui_core/datetime/nui_dt_util.dart';
import 'package:nui_core/model/enums/gradient_orientation.dart';
import 'package:nui_core/model/hex_color.dart';
import 'package:nui_core/string/nui_string_util.dart';
import 'package:nui_form/component/nuif_custom.dart';
import 'package:nui_form/component/nuif_dropdown.dart';
import 'package:nui_form/component/nuif_row.dart';
import 'package:nui_form/component/nuif_text.dart';
import 'package:nui_form/model/nuif_models.dart';
import 'package:nui_form/nuif_form.dart';

class IFRenderEngine{
  IFRenderEngine._builder(IFRenderEngineBuilder builder){
    this._body = builder._body;
    this._bodyJson = builder._bodyJson;
    this._textStyle = builder._textStyle;
    this._radioStyle = builder._radioStyle;
    this._radioDisabledStyle = builder._radioDisabledStyle;
    this._checkBoxStyle = builder._checkBoxStyle;
    this._checkBoxDisabledStyle = builder._checkBoxDisabledStyle;
    this._buttonStyle1 = builder._buttonStyle1;
    this._buttonStyle2 = builder._buttonStyle2;
    this._buttonDisabledStyle1 = builder._buttonDisabledStyle1;
    this._buttonDisabledStyle2 = builder._buttonDisabledStyle2;
  }

  String _bodyJson;
  IFBodyJson _body;
  NUIFCheckboxStyle _checkBoxStyle;
  NUIFCheckboxStyle _checkBoxDisabledStyle;
  NUIFRadioStyle _radioStyle;
  NUIFRadioStyle _radioDisabledStyle;
  NUIFTextStyle _textStyle;
  NUIFButtonStyle _buttonStyle1;
  NUIFButtonStyle _buttonDisabledStyle1;
  NUIFButtonStyle _buttonStyle2;
  NUIFButtonStyle _buttonDisabledStyle2;

  NUIFCheckboxStyle checkboxEnabledStyle() => _checkBoxStyle ?? NUIFCheckboxStyle(
    textStyle: TextStyle(fontSize: 14, color: NUIColors.NUITextGray),
    subTextStyle: TextStyle(fontSize: 12, color: NUIColors.NUITextGray),
    primaryIcon: Icon(Icons.check_box_outline_blank, color: NUIColors.NUITextGray),
    primaryIconChecked: Icon(Icons.check_box, color: NUIColors.NUIAccent)
  );

  NUIFCheckboxStyle checkboxDisbledStyle() => _checkBoxDisabledStyle ?? NUIFCheckboxStyle(
    textStyle: TextStyle(fontSize: 14, color: NUIColors.NUITextGray),
    subTextStyle: TextStyle(fontSize: 12, color: NUIColors.NUITextGray),
    primaryIcon: Icon(Icons.check_box_outline_blank, color: NUIColors.NUITextGray),
    primaryIconChecked: Icon(Icons.check_box, color: NUIColors.NUITextGray)
  );

  NUIFRadioStyle radioEnabledStyle() => _radioStyle ?? NUIFRadioStyle(
    textStyle: TextStyle(fontSize: 14, color: NUIColors.NUITextGray),
    primaryIcon: Icon(Icons.radio_button_off, color: NUIColors.NUITextGray,),
    primaryIconChecked: Icon(Icons.radio_button_checked, color: NUIColors.NUIAccent,)
  );

  NUIFRadioStyle radioDisabledStyle() => _radioDisabledStyle ?? NUIFRadioStyle(
    textStyle: TextStyle(fontSize: 14, color: NUIColors.NUITextGray),
    primaryIcon: Icon(Icons.radio_button_off, color: NUIColors.NUITextGray,),
    primaryIconChecked: Icon(Icons.radio_button_checked, color: NUIColors.NUITextGray,)
  );

  NUIFTextStyle inputTextStyle() => _textStyle ?? NUIFTextStyle(
    labelStyle: TextStyle(fontSize: 14, color: NUIColors.NUITextGray),
    helperStyle: TextStyle(fontSize: 12, color: NUIColors.NUITextGray),
    errorHelperStyle: TextStyle(fontSize: 12, color: NUIColors.NUIAccent),
    border: OutlineInputBorder(borderSide: BorderSide(color: NUIColors.NUITextGray, width: 2)),
    errorBorder: OutlineInputBorder(borderSide: BorderSide(color: NUIColors.NUIAccent, width: 2)),
    focusedBorder: OutlineInputBorder(borderSide: BorderSide(color: NUIColors.NUIAccent, width: 2)),
    design: NUIFTextDesign.BORDER,
    dense: true,
    cursor: NUIFTextCursor(color: NUIColors.NUITextBlack, cursorWidth: 2),
    textStyle: TextStyle(fontSize: 15, color: NUIColors.NUITextBlack),
    hintStyle: TextStyle(fontSize: 15, color: NUIColors.NUITextGray),
  );

  NUIFButtonStyle buttonStyle1() => _buttonStyle1 ?? NUIFButtonStyle(
    normalColor: NUIColors.NUIAccent,
    design: NUIFTButtonDesign.FILL,
    rippleColor: NUIColors.NUIShadowBlackTransparent,
    radius: 30,
    borderRadius: BorderRadius.circular(30),
    textStyle: TextStyle(fontSize: 14, color: NUIColors.NUIWhite),
    textAlign: TextAlign.center,
    padding: EdgeInsets.only(left: 20, right: 20, top: 10, bottom: 10)
  );

  NUIFButtonStyle buttonDisabledStyle1() => _buttonDisabledStyle1 ?? NUIFButtonStyle(
    normalColor: NUIColors.NUITransparent,
    design: NUIFTButtonDesign.OUTLINE,
    rippleColor: NUIColors.NUIShadowBlackTransparent,
    radius: 30,
    borderRadius: BorderRadius.circular(30),
    outlineColor: NUIColors.NUIGrayOutline,
    outlineThickness: 2,
    textStyle: TextStyle(fontSize: 14, color: NUIColors.NUIGrayOutline),
    textAlign: TextAlign.center,
    padding: EdgeInsets.only(left: 20, right: 20, top: 10, bottom: 10)
  );

  NUIFButtonStyle buttonStyle2() => _buttonStyle2 ?? NUIFButtonStyle(
      normalColor: NUIColors.NUIAccent,
      design: NUIFTButtonDesign.FILL,
      rippleColor: NUIColors.NUIShadowBlackTransparent,
      radius: 30,
      orientation: GradientOrientation.TL_BR,
      gradients: [Color(0xFFFFF5F5), NUIColors.NUIWhite],
      borderRadius: BorderRadius.circular(30),
      textStyle: TextStyle(fontSize: 14, color: NUIColors.NUIAccent, fontWeight: FontWeight.bold),
      textAlign: TextAlign.center,
      padding: EdgeInsets.only(left: 20, right: 20, top: 10, bottom: 10)
  );

  NUIFButtonStyle buttonDisabledStyle2() => _buttonDisabledStyle2 ?? NUIFButtonStyle(
      normalColor: NUIColors.NUITransparent,
      design: NUIFTButtonDesign.OUTLINE,
      rippleColor: NUIColors.NUIShadowBlackTransparent,
      radius: 30,
      borderRadius: BorderRadius.circular(30),
      outlineColor: NUIColors.NUIGrayOutline,
      outlineThickness: 2,
      textStyle: TextStyle(fontSize: 14, color: NUIColors.NUIGrayOutline),
      textAlign: TextAlign.center,
      padding: EdgeInsets.only(left: 20, right: 20, top: 10, bottom: 10)
  );

  NUIFTextInputType _maskedInputType(IFInputAllowType allowType){
    switch(allowType){
      case IFInputAllowType.Numeric1005: return NUIFTextInputType.NUMBER;
      case IFInputAllowType.AlphabetAZWithoutSpace1006: return NUIFTextInputType.NAME;
      case IFInputAllowType.AlphabetAZWithSpace1007: return NUIFTextInputType.NAME;
      case IFInputAllowType.Email1008: return NUIFTextInputType.EMAIL;
      case IFInputAllowType.Password1009: return NUIFTextInputType.PASSWORD_NEW;
      case IFInputAllowType.PasswordMask1010: return NUIFTextInputType.PASSWORD_NEW;
      case IFInputAllowType.Phone1011: return NUIFTextInputType.PHONE;
      case IFInputAllowType.Decimal1012: return NUIFTextInputType.DECIMAL;
      case IFInputAllowType.MsiaIC1013: return NUIFTextInputType.MALAYSIAN_IC;
      default: return null;
    }
  }
  
  TextInputType _inputType(IFInputAllowType allowType){
    switch(allowType){
      case IFInputAllowType.AlphanumericWithoutSpace1001: return TextInputType.text;
      case IFInputAllowType.AlphanumericWithSpace1002: return TextInputType.text;
      case IFInputAllowType.AlphabetAZAndSpecialChars1003: return TextInputType.text;
      case IFInputAllowType.AlphanumericWithSpecialChars1004: return TextInputType.text;
      case IFInputAllowType.Numeric1005: return TextInputType.numberWithOptions(decimal: false);
      case IFInputAllowType.AlphabetAZWithoutSpace1006: return TextInputType.name;
      case IFInputAllowType.AlphabetAZWithSpace1007: return TextInputType.name;
      case IFInputAllowType.Email1008: return TextInputType.emailAddress;
      case IFInputAllowType.Password1009: return TextInputType.visiblePassword;
      case IFInputAllowType.PasswordMask1010: return TextInputType.visiblePassword;
      case IFInputAllowType.Phone1011: return TextInputType.phone;
      case IFInputAllowType.Decimal1012: return TextInputType.numberWithOptions(decimal: true);
      case IFInputAllowType.MsiaIC1013: return TextInputType.numberWithOptions(decimal: false);
      default: return null;
    }
  }

  EdgeInsets _renderMargin(IFCommonJson json){
    return EdgeInsets.only(
      top: json.topSpacing?.toDouble() ?? 0,
      bottom: json.bottomSpacing?.toDouble() ?? 0,
      left: json.leftSpacing?.toDouble() ?? 0,
      right: json.rightSpacing?.toDouble() ?? 0
    );
  }

  NUIFRowItem _renderFormRow(Map<String, dynamic> child, NUIFRowItem row, NUIFColumnItem column, NUIForm form){
    final compBase = asObject<IFChildElement>(IFChildElementEnt(), child);
    final comp = _componentFromIFJson(child, form);
    if(comp == null) return row;

    var curRow = row ?? NUIFRowItem(items: List(), inpId: randomUUID());

    final intAlign = compBase.layout;
    if(intAlign == IFLayoutAlign.FULL_ROW.value()){
      curRow.items.add(comp);
      column.items.add(curRow);
      return NUIFRowItem(items: List(), inpId: randomUUID());
    }
    else if(intAlign == IFLayoutAlign.WRAP.value()){
      curRow.items.add(comp);
      return curRow;
    }
    else if(intAlign == IFLayoutAlign.NEXT_LINE.value()){
      curRow.items.add(comp);
      column.items.add(curRow);
      return NUIFRowItem(items: List(), inpId: randomUUID());
    }
    else{
      curRow.items.add(comp);
      column.items.add(curRow);
      return NUIFRowItem(items: List(), inpId: randomUUID());
    }
  }

  NUIFormComponent _componentFromIFJson(Map<String, dynamic> json, NUIForm form){
    final type = json["type"];
    if(match(type, IFormCType.Label.value())) return _renderLabel(asObject<IFLabelJson>(IFLabelJsonEnt(), json));
    if(match(type, IFormCType.SingleLineText.value())) return _renderTextField(asObject<IFSingleLineTextJson>(IFSingleLineTextJsonEnt(), json), multiline: false);
    if(match(type, IFormCType.MultiLineText.value())) return _renderTextField(asObject<IFSingleLineTextJson>(IFSingleLineTextJsonEnt(), json), multiline: true, inputType: NUIFTextInputType.MULTILINE);
    if(match(type, IFormCType.MYIc.value())) return _renderTextField(asObject<IFSingleLineTextJson>(IFSingleLineTextJsonEnt(), json), inputType: NUIFTextInputType.MALAYSIAN_IC);
    if(match(type, IFormCType.Password.value())) return _renderTextField(asObject<IFSingleLineTextJson>(IFSingleLineTextJsonEnt(), json), inputType: NUIFTextInputType.PASSWORD_NEW);
    if(match(type, IFormCType.Address.value())) return _renderTextField(asObject<IFSingleLineTextJson>(IFSingleLineTextJsonEnt(), json), inputType: NUIFTextInputType.MULTILINE);
    if(match(type, IFormCType.DropDown.value())) return _renderDropDown(asObject<IFDropDownJson>(IFDropDownJsonEnt(), json));
    if(match(type, IFormCType.Phone.value())) return _renderPhoneField(asObject<IFSingleLineTextJson>(IFSingleLineTextJsonEnt(), json));
    if(match(type, IFormCType.MYPhone.value())) return _renderPhoneField(asObject<IFSingleLineTextJson>(IFSingleLineTextJsonEnt(), json), isMalaysian: true);
    if(match(type, IFormCType.CheckGroup.value())) return _renderCheckGroup(asObject<IFCheckGroupJson>(IFCheckGroupJsonEnt(), json));
    if(match(type, IFormCType.DatePicker.value())) return _renderDateTimePicker(asObject<IFDateTimePickerJson>(IFDateTimePickerJsonEnt(), json), form, inputType: NUIFTextInputType.DATE);
    if(match(type, IFormCType.TimePicker.value())) return _renderDateTimePicker(asObject<IFDateTimePickerJson>(IFDateTimePickerJsonEnt(), json), form, inputType: NUIFTextInputType.TIME);
    if(match(type, IFormCType.ButtonsRow.value())) return _renderButtonsRow(asObject<IFButtonRowJson>(IFButtonRowJsonEnt(), json));
    return null;
  }

  NUIForm renderForm({FormTextChangeListener textChangeListener, FormButtonClickListener buttonClickListener, FormItemSelectListener itemSelectListener}){
    IFBodyJson bodyContent;
    if(_body != null){
      bodyContent = _body;
    }
    else{
      bodyContent = asObject<IFBodyJson>(IFBodyJsonEnt(), jsonDecode(_bodyJson));
    }

    final String mainInpId = randomUUID();

    final form = NUIForm(
        allButtonClickListener: buttonClickListener,
        allTextChangeListener: textChangeListener,
        allItemSelectListener: itemSelectListener
    );

    final mainColumn = NUIFColumnItem(items: List(), inpId: mainInpId, divider: bodyContent.inputVerticalSpacing?.toDouble() ?? 0);
    NUIFRowItem curRow;
    bodyContent.inputs.forEach((element) {
      curRow = _renderFormRow(element, curRow, mainColumn, form);
    });
    form.item = mainColumn;
    logNUI("IFormRenderEngine", "Total of ${mainColumn.items.length} in the form");
    return form;
  }

  NUIFCustomItem _renderLabel(IFLabelJson json, {bool alignVerticalCenter: false}){
    final textStyle = TextStyle(
        fontSize: json.fontSize?.toDouble() ?? 14,
        color: HexColor(json.fontHexColor ?? "000000"),
        fontStyle: json.fontBoldItalic == IFFontBoldItalic.BOTH.value() || json.fontBoldItalic == IFFontBoldItalic.ITALIC.value() ? FontStyle.italic : FontStyle.normal,
        fontWeight: json.fontBoldItalic == IFFontBoldItalic.BOTH.value() || json.fontBoldItalic == IFFontBoldItalic.BOLD.value() ? FontWeight.bold : FontWeight.normal
    );
    return NUIFCustomItem(
      inpId: json.inpId ?? randomUUID(),
      margin: _renderMargin(json),
      show: json.show,
      width: json.preferredWidth?.toDouble() ?? null,
      weight: json.preferredWidth == null ? 1 : null,
      align: json.horizontalAlign == IFHorizontalAlign.LEFT.value() ? Alignment.topLeft : json.horizontalAlign == IFHorizontalAlign.RIGHT.value() ? Alignment.topRight : Alignment.center,
      item: alignVerticalCenter != true ? Text(json.caption ?? "Label", style: textStyle, overflow: TextOverflow.ellipsis) : Align(alignment: Alignment.centerLeft, child: Text(json.caption ?? "Label", style: textStyle, overflow: TextOverflow.ellipsis))
    );
  }

  NUIFDropdownItem _renderDropDown(IFDropDownJson json){
    List<IFChildElement> selections = json.getChildElements();

    return NUIFDropdownItem(
      inpId: json.inpId ?? randomUUID(),
      hint: json.otherTxtPlaceHolder ?? json.caption ?? "Hint Missing",
      label: json.caption ?? "Title Missing",
      selections: selections,
      margin: _renderMargin(json),
      weight: json.preferredWidth == null ? 1 : null,
      style: inputTextStyle(),
    );
  }

  NUIFButtonStyle _buttonStyle(IFButtonStyle style){
    if(style == null) return null;

    return NUIFButtonStyle(
      rippleColor: !isNullOrEmpty(style.rippleHexColor) ? HexColor(style.rippleHexColor) : IFButtonStyle.DEFAULT_RIPPLE_COLOR,
      normalColor: !isNullOrEmpty(style.normalHexColor) ? HexColor(style.normalHexColor) : Colors.redAccent,
      design: style.isOutline() ? NUIFTButtonDesign.OUTLINE : NUIFTButtonDesign.FILL,
      outlineColor: !isNullOrEmpty(style.outlineHexColor) ? HexColor(style.outlineHexColor) : Colors.transparent,
      outlineThickness: style.outlineThickness ?? 0,
      radius: style.radius ?? 5,
      textStyle: TextStyle(fontSize: style.fontSize?.toDouble() ?? 14, color: !isNullOrEmpty(style.fontHexColor) ? HexColor(style.fontHexColor) : Colors.black),
      elevation: style.elevation ?? 0,
      gradients: !isNullOrEmpty(style.gradientHexColors) ? style.gradientHexColors.map((e) => HexColor(e)).toList() : null,
      padding: EdgeInsets.only(top: style.topPadding??0, bottom: style.bottomPadding??0, left: style.leftPadding??0, right: style.rightPadding??0)
    );
  }

  NUIFormComponent _renderButtonsRow(IFButtonRowJson json){
    List<IFButtonItemJson> selections = json.getChildElements();
    final isHorizontal = json.intLayout == IFOrientationAlign.HORIZONTAL.value();

    int index = 0;
    final buttons = selections.map((e){
      final enabledStyle = _buttonStyle(e.enabledStyle) ?? e.eleType == 1 ? buttonStyle1() : buttonStyle2();
      final disabledStyle = _buttonStyle(e.disabledStyle) ?? buttonDisabledStyle1();
      index ++;
      return NUIFButtonItem(inpId: e.inpId ?? json.getId() + "_$index", enabled: e.readOnly != true, text: e.caption, enabledStyle: enabledStyle, disabledStyle: disabledStyle, onTap: (){});
    }).toList();

    if(isHorizontal){
      return NUIFRowItem(
        padding: EdgeInsets.only(top: json.topSpacing?.toDouble()??0, bottom: json.bottomSpacing?.toDouble()??0, left: json.leftSpacing?.toDouble()??0, right: json.rightSpacing?.toDouble()??0),
        inpId: randomUUID(),
        divider: json.spaceBetIntHoriComponent ?? 10,
        items: buttons
      );
    }
    else{
      return NUIFColumnItem(
          padding: EdgeInsets.only(top: json.topSpacing?.toDouble()??0, bottom: json.bottomSpacing?.toDouble()??0, left: json.leftSpacing?.toDouble()??0, right: json.rightSpacing?.toDouble()??0),
          inpId: randomUUID(),
          divider: json.spaceBetIntHoriComponent ?? 10,
          items: buttons
      );
    }
  }

  NUIFormComponent _renderCheckGroup(IFCheckGroupJson json){
    List<Map<String, dynamic>> selections = json.childElements;
    bool isHorizontal = json.intLayout == 2;
    bool isRadio = json.asRadioButton == true || json.singleSelection == true;

    final List<IFChildElement> finalSelections = List();
    for(Map<String, dynamic> child in selections){
      IFChildElement ifChild = asObject(IFChildElementEnt(), child);
      IFSingleLineTextJson textChild = asObject(IFSingleLineTextJsonEnt(), child);
      if(!ifChild.otherTxtPlaceHolder.isNullOrEmpty()){
        final textItem = _renderTextField(textChild);
        textItem.inpId = ifChild.inpId ?? json.inpId + "_otherTxt";
        textItem.label = ifChild.otherTxtPlaceHolder ?? "Others";
        textItem.hint = ifChild.otherTxtPlaceHolder ?? "Others";
        textItem.compulsory = ifChild.selected == true && ifChild.compulsory == true;
        textItem.width = json.chkGroupTxtBoxPreferredWidth?.toDouble() ?? 120;
        textItem.weight = json.chkGroupTxtBoxPreferredWidth == null ? 1 : null;
        textItem.show = json.show != false;
        ifChild.otherText = textItem;
      }
      finalSelections.add(ifChild);
    }

    if(isRadio) {
      return NUIFRadioItem(
        inpId: json.inpId ?? randomUUID(),
        selections: finalSelections,
        margin: _renderMargin(json),
        weight: json.preferredWidth == null ? 1 : null,
        enabledStyle: radioEnabledStyle(),
        disabledStyle: radioDisabledStyle(),
        isHorizontal: isHorizontal,
      );
    }
    else{
      return NUIFCheckGroupItem(
        inpId: json.inpId ?? randomUUID(),
        selections: finalSelections,
        margin: _renderMargin(json),
        divider: json.spaceBetIntHoriComponent?.toDouble() ?? 0.0,
        weight: json.preferredWidth == null ? 1 : null,
        enabledStyle: checkboxEnabledStyle(),
        disabledStyle: checkboxDisbledStyle(),
        isHorizontal: isHorizontal,
      );
    }
  }

  NUIFRowItem _renderPhoneField(IFSingleLineTextJson json, {bool isMalaysian: false}){
    final errorMessages = NUIFValidatable.defaultErrorMessages();
    if(!isNullOrEmpty(json.validateErrMsg)){
      errorMessages[TextValidation.REG_EXPRESSION_FAILED] = json.validateErrMsg;
    }

    final String inpId = json.inpId ?? randomUUID();
    List<IFChildElement> countryCodes = json.getChildElements();

    final countryField = NUIFDropdownItem(
        inpId: inpId + "_cc",
        hint: "Country",
        label: "Country",
        selections: !isNullOrEmpty(countryCodes) ? countryCodes : [IFNormalSelection(title: "+60", id: "MY", value: "+60")],
        style: inputTextStyle(),
        weight: 0.25,
        showArrow: false,
        selectedItem: !isNullOrEmpty(countryCodes) ? null : IFNormalSelection(title: "+60", id: "MY", value: "+60"),
    );

    final numberField = NUIFTextItem(
        controller: TextEditingController(text: ""),
        inpId: inpId + "_no",
        label: json.caption ?? "Title Forgotten",
        hint: json.otherTxtPlaceHolder ?? "",
        compulsory: json.compulsory == true,
        margin: _renderMargin(json),
        regExp: json.inputAllowRegularExpression ?? inputAllowTypeValueOf(json.inputAllowType)?.regExp() ?? null,
        minLength: json.minLen ?? 0,
        maxLength: json.maxLen,
        errorMessages: errorMessages,
        width: json.preferredWidth?.toDouble() ?? null,
        show: json.show != false,
        enabled: json.readOnly != true,
        inputType: TextInputType.phone,
        maskedInputType: NUIFTextInputType.PHONE,
        style: inputTextStyle(),
        weight: 0.75
    );

    if(isMalaysian == true){
      //Format phone number for Malaysia
      numberField.postTextFormatter = (value, inputType, handler){
        final number = value;
        var maxNumber = 11;
        String newText;

        String text = value;
        if(text.length < handler.getNewText().length) { // handling backspace in keyboard
          newText = text;
        }
        else if (!isNullOrEmpty(text) && !match(text, handler.getNewText())) { // handling typing new characters.
          String tempText = text.replaceAll("-", "");
          if (number.startsWith("10") ||
              number.startsWith("12") ||
              number.startsWith("13") ||
              number.startsWith("14") ||
              number.startsWith("15") ||
              number.startsWith("16") ||
              number.startsWith("17") ||
              number.startsWith("18") ||
              number.startsWith("19")) {
            // it is 7 characters not including the prefix

            String firstSub = tempText.substring(0, 2);
            String secondSub = tempText.length > 2 ? tempText.substring(2, tempText.length) : "";

            newText = firstSub + "-" + secondSub;
            maxNumber = 7 + 2 + 1;
          } else if (number.startsWith("11")) {
            // it is 8 characters not including the prefix

            String firstSub = tempText.substring(0, 2);
            String secondSub = tempText.length > 2 ? tempText.substring(2, tempText.length) : "";

            newText = firstSub + "-" + secondSub;
            maxNumber = 8 + 2 + 1;
          } else {
            maxNumber = 8 + 1;
            newText = tempText;
          }
          if(!match(text, newText)) {
            handler.updateText(newText);
            final maxLengthChanged = numberField.maxLength != maxNumber;
            final minLengthChanged = numberField.minLength != maxNumber;
            if(maxLengthChanged || minLengthChanged){
              numberField.maxLength = maxNumber;
              numberField.minLength = maxNumber;
              handler.updateState();
            }
          }
        }
        return newText;
      };
    }

    return NUIFRowItem(items: [countryField, numberField], inpId: inpId, divider: 10);
  }

  //Rendering date time picker
  NUIFormComponent _renderDateTimePicker(IFDateTimePickerJson json, NUIForm form, {NUIFTextInputType inputType}){
    final errorMessages = NUIFValidatable.defaultErrorMessages();
    if(!isNullOrEmpty(json.validateErrMsg)){
      errorMessages[TextValidation.REG_EXPRESSION_FAILED] = json.validateErrMsg;
    }

    OnTextSubmitted task;
    final id = json.inpId ?? randomUUID();
    final containerId = randomUUID();

    final textItem = NUIFTextItem(
        controller: TextEditingController(text: ""),
        inpId: id,
        label: json.caption ?? "Title Forgotten",
        onChanged: (value){
          if(task != null) task(value);
        },
        hint: json.dateTimeDisplayFormat?.toLowerCase(),
        compulsory: json.compulsory == true,
        margin: _renderMargin(json),
        errorMessages: errorMessages,
        width: json.preferredWidth?.toDouble() ?? null,
        weight: json.preferredWidth == null ? 1 : null,
        show: json.show != false,
        enabled: json.readOnly != true,
        firstDate: json.getMinimumDate(),
        lastDate: json.getMaximumDate(),
        additionalFormat: json.dateTimeDisplayFormat,
        maskedInputType: inputType,
        style: inputTextStyle()
    );

    if(!json.childElements.isNullOrEmpty()){
      final validationElement = asObject(IFDateTimePickerJsonEnt(), json.childElements[0]);
      textItem.firstDate = textItem.firstDate ?? validationElement.getMinimumDate();
      textItem.lastDate = textItem.lastDate ?? validationElement.getMaximumDate();
      textItem.additionalFormat = textItem.additionalFormat ?? validationElement.dateTimeDisplayFormat;
    }

    NUIFRowItem rowItem;

    if(json.dateTimePickerType == IFDatePickerType.BIRTHDAY_AGE.value()){
      textItem.additionalFormat = NUIDTUtil.FORMAT_dd_MM_yyyy;

      final labelJson = IFLabelJson.empty();
      labelJson.inpId = id + "-age";
      labelJson.caption = "0";
      labelJson.show = false;
      labelJson.fontHexColor = "000000";
      labelJson.fontSize = 16;
      labelJson.preferredWidth = 40;
      labelJson.topSpacing = 15;
      labelJson.verticalAlign = IFVerticalAlign.CENTER.value();

      final labelItem = _renderLabel(labelJson, alignVerticalCenter: true);
      task = (value){
        try{
          DateTime dateAge = stringToDate(value);
          DateTime now = DateTime.now();
          final age = now.year - dateAge.year;

          if(labelJson.caption.match(age.toString())) return;

          labelJson.caption = age.toString();
          labelJson.show = true;

          NUIAsync.delay(Duration(milliseconds: 500), () {
            form.updateItem(containerId, (item) {
              rowItem.items = [textItem, _renderLabel(labelJson, alignVerticalCenter: true)];
              return rowItem;
            });
          });
        }catch(e){
          logNUI("IFRenderEngine", "Update with error : $e");
        }
      };

      rowItem = NUIFRowItem(items: [textItem, labelItem], inpId: containerId, divider: 10);
      return rowItem;
    }
    else if(json.dateTimePickerType == IFDatePickerType.DATE_RANGE.value()) {
      final inpId = textItem.inpId;

      textItem.label = "From";
      textItem.inpId = inpId + "_from";

      task = (value){
        try{
          DateTime selectedDate = stringToDate(value);

          NUIAsync.delay(Duration(milliseconds: 500), () {
            form.updateItem<NUIFTextItem>(inpId + "_to", (item) {
              if(item is NUIFTextItem){
                item.firstDate = selectedDate;
              }
              return item;
            });
          });
        }catch(e){
          logNUI("IFRenderEngine", "Update date range first date selection with error : $e");
        }
      };

      final textItem2 = NUIFTextItem(
          controller: TextEditingController(text: ""),
          inpId: inpId + "_to",
          label: "To",
          onChanged: (value){

          },
          hint: json.dateTimeDisplayFormat?.toLowerCase(),
          compulsory: json.compulsory == true,
          margin: _renderMargin(json),
          errorMessages: errorMessages,
          width: json.preferredWidth?.toDouble() ?? null,
          weight: json.preferredWidth == null ? 1 : null,
          show: json.show != false,
          enabled: json.readOnly != true,
          firstDate: json.getMinimumDate(),
          lastDate: json.getMaximumDate(),
          additionalFormat: json.dateTimeDisplayFormat,
          maskedInputType: inputType,
          style: inputTextStyle(),
      );

      if(!json.childElements.isNullOrEmpty()){
        final validationElement = asObject(IFDateTimePickerJsonEnt(), json.childElements[0]);
        textItem2.firstDate = textItem2.firstDate ?? validationElement.getMinimumDate();
        textItem2.lastDate = textItem2.lastDate ?? validationElement.getMaximumDate();
        textItem2.additionalFormat = textItem2.additionalFormat ?? validationElement.dateTimeDisplayFormat;
      }

      return NUIFRowItem(items: [textItem, textItem2], inpId: containerId, divider: 10);
    }
    else if(json.dateTimePickerType == IFDatePickerType.DATE_TIME.value()) {
      textItem.maskedInputType = NUIFTextInputType.DATETIME;
      textItem.inputType = TextInputType.datetime;
    }

    return textItem;
  }

  //Rendering single line text
  NUIFTextItem _renderTextField(IFSingleLineTextJson json, {NUIFTextInputType inputType, bool multiline: false}){
    final errorMessages = NUIFValidatable.defaultErrorMessages();
    if(!isNullOrEmpty(json.validateErrMsg)){
      logNUI("IFRenderEngine", "Found custom reg exp message : ${json.validateErrMsg}");
      errorMessages[TextValidation.REG_EXPRESSION_FAILED] = json.validateErrMsg;
    }

    final textItem = NUIFTextItem(
        controller: TextEditingController(text: ""),
        inpId: json.inpId ?? randomUUID(),
        label: json.caption ?? "Title Forgotten",
        hint: json.otherTxtPlaceHolder ?? "",
        compulsory: json.compulsory == true,
        margin: _renderMargin(json),
        regExp: json.inputAllowRegularExpression ?? inputAllowTypeValueOf(json.inputAllowType)?.regExp() ?? null,
        minLength: json.minLen ?? (inputType == NUIFTextInputType.MALAYSIAN_IC ? 14 : null),
        maxLength: json.maxLen,
        errorMessages: errorMessages,
        width: json.preferredWidth?.toDouble() ?? null,
        weight: json.preferredWidth == null ? 1 : null,
        show: json.show != false,
        minLine: json.minLines,
        maxLine: json.maxLines,
        validateOnChange: true,
        enabled: json.readOnly != true,
        inputType: _inputType(inputAllowTypeValueOf(json.inputAllowType)) ?? TextInputType.text,
        maskedInputType: inputType ?? _maskedInputType(inputAllowTypeValueOf(json.inputAllowType)),
        style: inputTextStyle()
    );

    if(!json.childElements.isNullOrEmpty()){
      final validationElement = asObject(IFSingleLineTextJsonEnt(), json.childElements[0]);
      textItem.minLength = json.minLen ?? validationElement.minLen ?? 0;
      textItem.maxLength = json.maxLen ?? validationElement.maxLen;
      textItem.regExp = textItem.regExp ?? validationElement.inputAllowRegularExpression ?? inputAllowTypeValueOf(validationElement.inputAllowType)?.regExp() ?? null;
      textItem.inputType = _inputType(inputAllowTypeValueOf(json.inputAllowType)) ?? _inputType(inputAllowTypeValueOf(validationElement.inputAllowType)) ?? TextInputType.text;
    }

    return textItem;
  }

}

class IFRenderEngineBuilder{
  String _bodyJson;
  IFBodyJson _body;
  NUIFCheckboxStyle _checkBoxStyle;
  NUIFCheckboxStyle _checkBoxDisabledStyle;
  NUIFRadioStyle _radioStyle;
  NUIFRadioStyle _radioDisabledStyle;
  NUIFTextStyle _textStyle;
  NUIFButtonStyle _buttonStyle1;
  NUIFButtonStyle _buttonDisabledStyle1;
  NUIFButtonStyle _buttonStyle2;
  NUIFButtonStyle _buttonDisabledStyle2;

  IFRenderEngineBuilder bodyJson(String json){
    this._bodyJson = json;
    return this;
  }

  IFRenderEngineBuilder body(IFBodyJson body){
    this._body = body;
    return this;
  }

  IFRenderEngineBuilder checkboxStyle(NUIFCheckboxStyle style){
    this._checkBoxStyle = style;
    return this;
  }

  IFRenderEngineBuilder checkboxDisabledStyle(NUIFCheckboxStyle style){
    this._checkBoxDisabledStyle = style;
    return this;
  }

  IFRenderEngineBuilder radioStyle(NUIFRadioStyle style){
    this._radioStyle = style;
    return this;
  }

  IFRenderEngineBuilder radioDisabledStyle(NUIFRadioStyle style){
    this._radioDisabledStyle = style;
    return this;
  }

  IFRenderEngineBuilder textStyle(NUIFTextStyle style){
    this._textStyle = style;
    return this;
  }

  IFRenderEngineBuilder buttonStyle1(NUIFButtonStyle style){
    this._buttonStyle1 = style;
    return this;
  }

  IFRenderEngineBuilder buttonDisabledStyle1(NUIFButtonStyle style){
    this._buttonDisabledStyle1 = style;
    return this;
  }

  IFRenderEngineBuilder buttonStyle2(NUIFButtonStyle style){
    this._buttonStyle2 = style;
    return this;
  }

  IFRenderEngineBuilder buttonDisabledStyle2(NUIFButtonStyle style){
    this._buttonDisabledStyle2 = style;
    return this;
  }

  IFRenderEngine build(){
    return IFRenderEngine._builder(this);
  }
}