
import 'dart:ui';

import 'package:nui_core/data/nui_ent.dart';
import 'package:nui_core/nui_core.dart';

class IFButtonStyle extends NUIEnt<IFButtonStyle>{
  static const String DESIGN_OUTLINE = "outline";
  static const String DESIGN_FILL = "fill";
  static const String BR_TL = "BR_TL";
  static const String BL_TR = "BL_TR";
  static const String TL_BR = "TL_BR";
  static const String TR_BL = "TR_BL";
  static const String L_R = "L_R";
  static const String R_L = "R_L";
  static const String T_B = "T_B";
  static const String B_T = "B_T";
  static const Color DEFAULT_RIPPLE_COLOR = Color(0x30000000);
  IFButtonStyle.empty();

  int fontSize;
  int fontBoldItalic;
  String fontHexColor;
  double topPadding;
  double bottomPadding;
  double leftPadding;
  double rightPadding;
  String rippleHexColor;
  String normalHexColor;
  String outlineHexColor;
  double outlineThickness;
  String design; //outline, fill
  double radius;
  double elevation;
  double borderRadius;
  List<String> gradientHexColors;
  String gradientOrientation; // BR_TL, BL_TR, TL_BR, TR_BL, L_R, R_L, T_B, B_T

  @override
  NUIEntMapper<IFButtonStyle> mapper() => IFButtonStyleEnt();

  bool isOutline(){
    return match(design, DESIGN_OUTLINE);
  }
  bool isFill(){
    return match(design, DESIGN_FILL);
  }
}
class IFButtonStyleEnt extends NUIEntMapper<IFButtonStyle>{
  @override
  String entityName() => "NUIFButtonStyle";

  @override
  List<NUIEntField> fields() {
    return [
      NUIEntField<IFButtonStyle>(name: "fontSize", type: NUIEntType.INTEGER, setter: (data, value) => data.fontSize = value, getter: (data) => data.fontSize),
      NUIEntField<IFButtonStyle>(name: "fontHexColor", type: NUIEntType.STRING, setter: (data, value) => data.fontHexColor = value, getter: (data) => data.fontHexColor),
      NUIEntField<IFButtonStyle>(name: "fontBoldItalic", type: NUIEntType.INTEGER, setter: (data, value) => data.fontBoldItalic = value, getter: (data) => data.fontBoldItalic),
      NUIEntField<IFButtonStyle>(name: "topPadding", type: NUIEntType.INTEGER, setter: (data, value) => data.topPadding = value, getter: (data) => data.topPadding),
      NUIEntField<IFButtonStyle>(name: "bottomPadding", type: NUIEntType.INTEGER, setter: (data, value) => data.bottomPadding = value, getter: (data) => data.bottomPadding),
      NUIEntField<IFButtonStyle>(name: "leftPadding", type: NUIEntType.INTEGER, setter: (data, value) => data.leftPadding = value, getter: (data) => data.leftPadding),
      NUIEntField<IFButtonStyle>(name: "rightPadding", type: NUIEntType.INTEGER, setter: (data, value) => data.rightPadding = value, getter: (data) => data.rightPadding),
      NUIEntField<IFButtonStyle>(name: "rippleHexColor", type: NUIEntType.STRING, setter: (data, value) => data.rippleHexColor = value, getter: (data) => data.rippleHexColor),
      NUIEntField<IFButtonStyle>(name: "normalHexColor", type: NUIEntType.STRING, setter: (data, value) => data.normalHexColor = value, getter: (data) => data.normalHexColor),
      NUIEntField<IFButtonStyle>(name: "outlineHexColor", type: NUIEntType.STRING, setter: (data, value) => data.outlineHexColor = value, getter: (data) => data.outlineHexColor),
      NUIEntField<IFButtonStyle>(name: "outlineThickness", type: NUIEntType.DOUBLE, setter: (data, value) => data.outlineThickness = value, getter: (data) => data.outlineThickness),
      NUIEntField<IFButtonStyle>(name: "design", type: NUIEntType.STRING, setter: (data, value) => data.design = value, getter: (data) => data.design),
      NUIEntField<IFButtonStyle>(name: "radius", type: NUIEntType.DOUBLE, setter: (data, value) => data.radius = value, getter: (data) => data.radius),
      NUIEntField<IFButtonStyle>(name: "elevation", type: NUIEntType.DOUBLE, setter: (data, value) => data.elevation = value, getter: (data) => data.elevation),
      NUIEntField<IFButtonStyle>(name: "borderRadius", type: NUIEntType.DOUBLE, setter: (data, value) => data.borderRadius = value, getter: (data) => data.borderRadius),
      NUIEntField<IFButtonStyle>(name: "gradientHexColors", type: NUIEntType.LIST_STRING, setter: (data, value) => data.gradientHexColors = asList<String>(value), getter: (data) => data.gradientHexColors),
      NUIEntField<IFButtonStyle>(name: "gradientOrientation", type: NUIEntType.STRING, setter: (data, value) => data.gradientOrientation = value, getter: (data) => data.gradientOrientation),
    ];
  }

  @override
  IFButtonStyle newInstance() => IFButtonStyle.empty();
}